

function PaletteViewModel (viewId, colorChangeDispatcher)
{
	var self = this;
	
	self.viewId = viewId;
	self.view = document.getElementById(viewId);
	
	self.colorChangeDispatcher = colorChangeDispatcher;
	
	
	// create
	//self.buttonDiv = getDescendantById(self.view, "ColorButtonDiv");
	//self.savePaletteButton = getDescendantById(self.view, "savePaletteButton");
    //self.savePaletteButton.addEventListener('click', self.dummy);  // assignment needs to be _after_ the function declaration
	
	self.autoAddColors = false;
	
	self.selectedColor = null;
		
	self.buttons = new Array();
	
	
	
	self.setAutoAddColors = function(autoAddColors)
	{
	    self.autoAddColors = autoAddColors;
	}
	
	
	self.addColorButtons = function(namedColorArray)
	{
		for (var i = 0; i < namedColorArray.length; i++)
		{
			self.addColorButton(namedColorArray[i]);
		}
	}
	
	
	self.addColorButton = function(namedColor)
    {
    	// create NamedColorButton with desired color as background, and name as tooltip
    	var namedColorButton = new NamedColorButton(namedColor);
    	
    	// set the omclick routine to our color-selected method
    	namedColorButton.button.onclick = self.setSelectedColorFromColorButton;
    	
    	// add to display and list of buttons
    	self.view.append(namedColorButton.button);
    	self.buttons.push(namedColorButton);
    	
    	return namedColorButton;
    }
	
	
	self.removeColorButton = function(namedColorButton)
    {
    	// remove from display and list of buttons
    	self.view.removeChild(namedColorButton.button);
    	self.buttons.splice(self.buttons.indexOf(namedColorButton), 1)
    }
	
	
	self.setSelectedColorFromColorButton = function(event)
    {
    	// event will be click from NamedColorButton.button
    	var button = event.target;
    	var namedColorButton = self.findNamedButton(button);
    	
    	self.setSelectedButton(namedColorButton);
    	
    	self.selectedColor = namedColorButton.namedColor;
    	
    	// inform listeners
    	self.colorChangeDispatcher.broadcastChange(self.selectedColor);
		
    }
	
	
	self.getSelectedColor = function()
	{
		return self.selectedColor;
	}
	
	
	self.findNamedButton = function(button)
	{
		for (var i = 0; i < self.buttons.length; i++)
		{
			if (button == self.buttons[i].button)
			{
				return self.buttons[i];
			}
		}
		
		return null;
	}
	
	
	self.setSelectedButton = function(namedColorButton)
	{
		for (var i = 0; i < self.buttons.length; i++)
		{
			self.buttons[i].setSelected(false);
		}
		
		if (namedColorButton != null)
		{
			namedColorButton.setSelected(true);
		}
	}
	
	
	self.onColorChange = function(namedColor)
	{
		self.selectedColor = namedColor;
		
		for (var i = 0; i < self.buttons.length; i++)
		{
			if (self.buttons[i].namedColor.equals(namedColor))
			{
				self.buttons[i].setSelected(true);
				return;
			}
			else
			{
				self.buttons[i].setSelected(false);
			}
		}
		
		// if get here, don't have it; add if we're adding colors automatically
		if (self.autoAddColors)
	    {
    		var newButton = self.addColorButton(namedColor);
    		newButton.setSelected(true);
	    }
		
	}
	
	self.colorChangeDispatcher.addChangeListener(self, self.onColorChange);
	
	
	
	/**
     * Instantiate Palette from XML document accessed by editor.
     * @param editor  XML document editor initialized with palette XML doc
     */
    self.initializeFromXml = function(/*Element*/ parent, /*XmlDocumentEditor*/ editor)
    {
        var paletteElement = editor.getElement(parent, "color_palette");
        
        var namedColorElements = editor.getElements(paletteElement, "named_color");
        
        for (var i = 0; i < namedColorElements.length; i++)
        {
            var name = namedColorElements[i].getAttribute("color_name");
            var valueString = namedColorElements[i].textContent;
            
            // value comes in as aarrggbb; change to just rrggbb with "#" prepended
            var rgbString = NamedColor.argbToColorString(valueString);
            var color = new Color(rgbString);
            
            var namedColor = NamedColor.getNamedColor(name, color);
            self.addColorButton(namedColor);
        }

    }

	
	
	self.writeXML = function(/*Element*/ parent, /*XmlDocumentEditor*/ editor)
	{
		// create top-level color palette node
        var paletteElement = editor.appendElement(parent, "color_palette");
        
        for (var i = 0; i < self.buttons.length; i++)
        {
            var name = self.buttons[i].namedColor.name;
            var namedColor = self.buttons[i].namedColor;
            var element = editor.appendHexElement(paletteElement, "named_color", namedColor.getARGBString());
            element.setAttribute("color_name", name);
        }
	}
	
	
	self.generateXmlString = function()
    {
		// stuff for generating XML documents
	    var paletteDocumentRootElementName = "rosette_designer_palette";
	    var paletteDocumentHeaderComment = "Color palette data for use with RosetteDesigner application";
		
		
		// create DOM document with the list of colors represented, and populate it
        var xmlDoc = new Document();
        var docHeaderComment = paletteDocumentHeaderComment;
        xmlDoc.createComment(docHeaderComment);
        
        // add the standard XML processing instruction <?xml version="1.0" encoding="UTF-8"?>
        const processingInstruction = xmlDoc.createProcessingInstruction('xml', 'version="1.0" encoding="UTF-8"');
        xmlDoc.appendChild(processingInstruction);
        
        // create a "root" element for the document
        var docElementName = paletteDocumentRootElementName;
        var docElement = xmlDoc.createElement(docElementName);
        xmlDoc.appendChild(docElement);
        
        // just a convenience wrapper that provides methods for simultaneously creating, assigning value to, and appending Elements
        var editor = new XmlDocumentEditor(xmlDoc);
        
        // write out palette information to document
        self.writeXML(docElement, editor);
        
        // return the string contents of the doc
        var serializer = new XMLSerializer();
        var xmlString = serializer.serializeToString(xmlDoc);

        console.log(xmlString);
        
        return xmlString;
    
    }

}