


// constants
	
//symmetry constants
Rosette.SLIP_MATCH = "slip_match";
Rosette.BOOK_MATCH = "book_match";
Rosette.INVERSION = "inversion";
Rosette.BOOK_INVERSION = "book_inversion";

Rosette.SLIP_MATCH_INDEX = 1;
Rosette.BOOK_MATCH_INDEX = 2;
Rosette.INVERSION_INDEX = 3;
Rosette.BOOK_INVERSION_INDEX = 4;

Rosette.SYMMETRY_ARRAY = [/*0 unused*/ "invalid", Rosette.SLIP_MATCH, Rosette.BOOK_MATCH, Rosette.INVERSION, Rosette.BOOK_INVERSION];

// default values
Rosette.DEFAULT_SYMMETRY_INDEX = Rosette.SLIP_MATCH_INDEX;
Rosette.DEFAULT_SYMMETRY = "slip_match";
Rosette.DEFAULT_RADIUS = 2.25;
Rosette.DEFAULT_TILE_WIDTH = 1.0;
Rosette.DEFAULT_TILE_HEIGHT = 1.0;
Rosette.DEFAULT_NUM_ROWS = 20;
Rosette.DEFAULT_NUM_COLUMNS = 20;



function Rosette(numRows, numCols, symmetry, tileWidth, tileHeight, radius, backColor, gridColor)

{
	var self = this;
	
	// initialization
	
	self.tileWidth = tileWidth;
	self.tileHeight = tileHeight;
	self.radius = radius;
	self.symmetry = symmetry;
	self.backColor = backColor;
	self.gridColor = gridColor;
	
	self.evenTile = new Tile(numRows, numCols, backColor, gridColor);
	self.oddTile = new Tile(numRows, numCols, backColor, gridColor);
	
	self.innerRings = new Array();
	self.outerRings = new Array();
	
	
	
	self.getRows = function()
	{
		return self.evenTile.numRows;
	}
	
	
	self.getCols = function()
	{
		return self.evenTile.numCols;
	}
	
	
	self.getTileHeight = function()
	{
		return self.tileHeight;
	}
	
	
	self.getTileWidth = function()
	{
		return self.tileWidth;
	}
	
	
	self.getSymmetry = function()
	{
		return self.symmetry;
	}
	
	
	self.getRadius = function()
	{
		return self.radius;
	}
	
	
	// includes border rings if any
	self.getOuterRadius = function()
    {
        var ringWidth = 0;
        for (var i = 0; i < self.outerRings.length; i++)
        {
            ringWidth += self.outerRings[i].width;
        }
	    
	    return self.radius + self.tileHeight + ringWidth;
    }
	
	
	self.getElementColor = function(row, column)
    {
        return self.evenTile.element[row][column];
    }
	
	
	self.getEvenElementColor = function(row, column)
	{
		return self.evenTile.element[row][column];
	}
	
	
	self.getOddElementColor = function(row, column)
	{
		return self.oddTile.element[row][column];
	}
	
	
	self.getAdjustedRowColumn = function(tileNum, row, column)
	{
		var oddRow = row;
		var oddColumn = column;
		
		if (tileNum % 2 != 0)	// odd tile
		{
			switch (self.symmetry)
	        {
	            case Rosette.BOOK_MATCH:
	            {
	            	oddColumn = self.evenTile.numCols - column - 1;
	                break;
	            }
	            case Rosette.INVERSION:
	            {
	            	oddRow = self.evenTile.numRows - row - 1;
	                break;
	            }
	            case Rosette.BOOK_INVERSION:
	            {
	            	oddRow = self.evenTile.numRows - row - 1;
	                oddColumn = self.evenTile.numCols - column - 1;
	                break;
	            }
	            case Rosette.SLIP_MATCH:
	            default:
	            {
	                // no alteration needed
	                break;
	            }
	        }
		}
		
		return {row: oddRow, column: oddColumn};
	}
	
	
	self.setSymmetry = function(symmetry)
	{
		// check to see if it's a valid symmetry spec
		switch (symmetry)
		{
			
			case Rosette.SLIP_MATCH:
			case Rosette.BOOK_MATCH:
			case Rosette.INVERSION:
			case Rosette.BOOK_INVERSION:
			{
				self.symmetry = symmetry;
				break;
			}
			
			default:
			{
				self.symmetry = Rosette.SLIP_MATCH;
				break;
			}

		}
		
		// rebuild odd tile
		self.oddTile.initializeFromBaseTile(self.evenTile, self.symmetry);
	}
	
	
	self.setRows = function(rows)
	{
		self.evenTile.setRows(rows);
		
		// rebuild odd tile
        self.oddTile.initializeFromBaseTile(self.evenTile, self.symmetry);
	}
	
	
	self.setCols = function(cols)
	{
		self.evenTile.setCols(cols);

		// rebuild odd tile
        self.oddTile.initializeFromBaseTile(self.evenTile, self.symmetry);
	}
	
	
	self.setTileHeight = function(height)
	{
		self.tileHeight = height;
	}
	
	
	self.setTileWidth = function(width)
	{
		self.tileWidth = width;
	}
	
	
	self.setRadius = function(radius)
	{
		self.radius = radius;
	}
	
	
	self.setTileElementColor = function(row, column, color)
	{
		if ((row >= self.evenTile.numRows) || (column >= self.evenTile.numCols))
			return;
				
		self.evenTile.element[row][column] = color;
		
		// set color in appropriate element of odd tile
		switch (self.symmetry)
        {
            case Rosette.SLIP_MATCH:
            {
                oddRow = row;
                oddColumn = column;
                break;
            }
            case Rosette.BOOK_MATCH:
            {
                oddRow = row;
                oddColumn = self.evenTile.numCols - column - 1;
                break;
            }
            case Rosette.INVERSION:
            {
                oddRow = self.evenTile.numRows - row - 1;
                oddColumn = column;
                break;
            }
            case Rosette.BOOK_INVERSION:
            {
                oddRow = self.evenTile.numRows - row - 1;
                oddColumn = self.evenTile.numCols - column - 1;
                break;
            }
            default:
            {
                oddRow = row;
                oddColumn = column;
                break;
            }
        }
        
        // set color appropriately
        self.oddTile.element[oddRow][oddColumn] = color;
	}
	
	
	self.shiftVertically = function(shift)
    {
        self.evenTile.shiftVertically(shift);
        self.oddTile.initializeFromBaseTile(self.evenTile, self.symmetry);
    }
    
    
	self.shiftHorizontally = function(shift)
    {
        self.evenTile.shiftHorizontally(shift);
        self.oddTile.initializeFromBaseTile(self.evenTile, self.symmetry);
    }
	
	
	self.addInnerRing = function(width, namedColor)
	{
	    var ring = new BorderRing(width, namedColor);
	    self.innerRings.push(ring);
	}
	
	
	self.addOuterRing = function(width, namedColor)
    {
        var ring = new BorderRing(width, namedColor);
        self.outerRings.push(ring);
    }
	
	
	self.removeInnerRing = function()
    {
        self.innerRings.pop();
    }
	
	
	self.removeOuterRing = function()
    {
        self.outerRings.pop();
    }
	
	
	self.clear = function()
	{
	    // set tile elements to backcolor
	    self.evenTile.clear(self.backColor);
	    self.oddTile.clear(self.backColor);
	}
	
	
	
	/**
     * Instantiate Rosette from XML document accessed by editor.
     * @param editor  XML document editor initialized with rosette XML doc
     */
    self.initializeFromXml = function(/*Element*/ parent, /*XmlDocumentEditor*/ editor)
    {
        var rosetteElement = editor.getElement(parent, "rosette");
        
        var symmetryIndex = editor.getIntElementValue(rosetteElement, "symmetry", Rosette.DEFAULT_SYMMETRY_INDEX);
        self.symmetry = Rosette.SYMMETRY_ARRAY[symmetryIndex];
        
        self.tileWidth = editor.getDoubleElementValue(rosetteElement, "tile_width", Rosette.DEFAULT_TILE_WIDTH);
        self.tileHeight = editor.getDoubleElementValue(rosetteElement, "tile_height", Rosette.DEFAULT_TILE_HEIGHT);
        self.radius = editor.getDoubleElementValue(rosetteElement, "radius", Rosette.DEFAULT_RADIUS);
        
        // validate the values supplied: must be positive...
        if ((self.radius <= 0) || (self.tileWidth <= 0) || (self.tileHeight <= 0))
            throw new InvalidParameterException("Rosette radius, height and width must be greater than 0");
            
        // ...don't want them too large or will crash (too much memory)...
        if ((self.tileWidth > self.maxTileHeightWidth) || (self.tileHeight > self.maxTileHeightWidth))
            throw new InvalidParameterException("Tile height and width must be less than " + self.maxTileHeightWidth);

        // initialize even tile from XML, and odd tile from even tile
        self.evenTile.initializeFromXml(rosetteElement, editor);
        self.oddTile.initializeFromBaseTile(self.evenTile, self.symmetry);
        
        // get border rings
        self.innerRings = new Array();
        var innerRingsElement = editor.getElement(rosetteElement, "inner_rings");
        if (innerRingsElement != null)
        {
            var numRings = editor.getIntElementValue(innerRingsElement, "num_rings", 0);
            for (var i = 0; i < numRings; i++)
            {
                // append each ring, indicating the ring number in the XML tag
                var ring = new BorderRing();
                ring.initializeFromXml(innerRingsElement, editor, "_" + i);
                self.innerRings.push(ring);
            }
        }
        
        self.outerRings = new Array();
        var outerRingsElement = editor.getElement(rosetteElement, "outer_rings");
        if (outerRingsElement != null)
        {
            var numRings = editor.getIntElementValue(outerRingsElement, "num_rings", 0);
            for (var i = 0; i < numRings; i++)
            {
                // append each ring, indicating the ring number in the XML tag
                var ring = new BorderRing();
                ring.initializeFromXml(outerRingsElement, editor, "_" + i);
                self.outerRings.push(ring);
            }
        }

    }
	
	
	
	
	/**
     * Generate XML representation of rosette
     * @return
     */
    self.writeXML = function(/*Element*/ parent, /*XmlDocumentEditor*/ editor)
    {
        // create top-level rosette node
        rosetteElement = editor.appendElement(parent, "rosette");
        
        // put in data
        editor.appendIntElement(rosetteElement, "symmetry", self.getSymmetryIndex(self.symmetry));
        editor.appendDoubleElement(rosetteElement, "tile_width", self.tileWidth);
        editor.appendDoubleElement(rosetteElement, "tile_height", self.tileHeight);
        editor.appendDoubleElement(rosetteElement, "radius", self.radius);

        // write out just even tile; can re-create odd tile from symmetry
        self.evenTile.writeXML(rosetteElement, editor);
        
        // write out inner and outer rings
        var innerRingsElement = editor.appendElement(rosetteElement, "inner_rings");
        editor.appendIntElement(innerRingsElement, "num_rings", self.innerRings.length);
        for (var i = 0; i < self.innerRings.length; i++)
        {
            // append each ring, indicating the ring number in the XML tag
            var ring = self.innerRings[i];
            ring.writeXML(innerRingsElement, editor, "_" + i);
        }
        

        var outerRingsElement = editor.appendElement(rosetteElement, "outer_rings");
        editor.appendIntElement(outerRingsElement, "num_rings", self.outerRings.length);
        for (var i = 0; i < self.outerRings.length; i++)
        {
         // append each ring, indicating the ring number in the XML tag
            var ring = self.outerRings[i];
            ring.writeXML(outerRingsElement, editor, "_" + i);
        }
        
    }
    
    
    self.getSymmetryIndex = function(symmetryString)
    {
    	for (var i = 0; i < Rosette.SYMMETRY_ARRAY.length; i++)
		{
    		if (symmetryString == Rosette.SYMMETRY_ARRAY[i])
    			return i;
		}
    	
    	return 0; // invalid
    }
	




}